/* PHAVerLite: PHAVer + PPLite.
   Copyright (C) 2019-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PHAVerLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <phaverlite-config.h>

#include "abs_poly.hh"

#include <iostream>
#include <string>

using namespace pplite;

// Definitions of explicit template instantiations.
template class poly_adapter<Poly>;
template class poly_adapter<Stats<Poly>>;
template class poly_adapter<F_Poly>;
template class poly_adapter<Stats<F_Poly>>;

Poly_Kind poly_kind = Poly_Kind::POLY;

bool set_poly_kind(const std::string& name, bool noisy) {
  for (auto i = 0; i != poly_kind_size; ++i) {
    if (name == poly_kind_names[i]) {
      poly_kind = static_cast<Poly_Kind>(i);
      const bool requires_stats = (name.find("Stats") != std::string::npos);
      pplite::set_noisy_stats(requires_stats);
      if (noisy) {
        std::string msg = "Poly kind set to " + name;
        std::string marker = std::string(msg.size(), '=');
        std::cout << marker
                  << "\n" << msg << "\n"
                  << marker
                  << std::endl;
      }
      return true;
    }
  }
  if (noisy) {
    std::string msg = "Unknown poly kind name '" + name +"'.\n";
    msg += "Valid values:";
    bool comma = false;
    for (const auto& name : poly_kind_names) {
      msg += (comma ? ", " : " ") + std::string(name);
      comma = true;
    }
    std::cout << msg << std::endl;
  }
  return false;
}
